@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.36 2003/05/27 20:01:49 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.44 2003/06/22 19:11:28 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -103,6 +103,15 @@ static int speeds[2] = {
 	XINE_SPEED_NORMAL,
 };
 
+static int video_props[2] = {
+	XINE_PARAM_VO_BRIGHTNESS,
+	XINE_PARAM_VO_CONTRAST,
+};
+static char *video_props_str[2] = {
+	GCONF_PREFIX"/brightness",
+	GCONF_PREFIX"/contrast",
+};
+
 struct BaconVideoWidgetPrivate {
 	/* Xine stuff */
 	xine_t *xine;
@@ -114,6 +123,7 @@ struct BaconVideoWidgetPrivate {
 	gboolean started;
 
 	/* Configuration */
+	GConfClient *gc;
 	gboolean null_out;
 	char *mrl;
 
@@ -138,6 +148,7 @@ struct BaconVideoWidgetPrivate {
 	GdkPixbuf *icon;
 	gint volume;
 	TvOutType tvout;
+	guint32 video_fcc, audio_fcc;
 
 	GAsyncQueue *queue;
 	int video_width, video_height;
@@ -286,8 +297,8 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 				G_SIGNAL_RUN_LAST,
 				G_STRUCT_OFFSET (BaconVideoWidgetClass, error),
 				NULL, NULL,
-				g_cclosure_marshal_VOID__STRING,
-				G_TYPE_NONE, 1, G_TYPE_STRING);
+				baconvideowidget_marshal_VOID__STRING_BOOLEAN,
+				G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_BOOLEAN);
 
 	bvw_table_signals[EOS] =
 		g_signal_new ("eos",
@@ -349,7 +360,6 @@ bacon_video_widget_instance_init (BaconVideoWidget *bvw)
 	GtkWidget *widget = (GtkWidget *) bvw;
 	const char *const *autoplug_list;
 	int i = 0;
-	GConfClient *gc;
 
 	g_return_if_fail (bvw != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
@@ -373,8 +383,8 @@ bacon_video_widget_instance_init (BaconVideoWidget *bvw)
 	xine_init (bvw->priv->xine);
 
 	/* Debug configuration */
-	gc = gconf_client_get_default ();
-	if (gconf_client_get_bool (gc, GCONF_PREFIX"/debug", NULL) == FALSE)
+	bvw->priv->gc = gconf_client_get_default ();
+	if (gconf_client_get_bool (bvw->priv->gc, GCONF_PREFIX"/debug", NULL) == FALSE)
 	{
 		xine_engine_set_param (bvw->priv->xine,
 				XINE_ENGINE_PARAM_VERBOSITY,
@@ -614,7 +624,6 @@ load_audio_out_driver (BaconVideoWidget *bvw, GError **err)
 static void
 setup_config (BaconVideoWidget *bvw)
 {
-	GConfClient *gc;
 	char *path;
 	xine_cfg_entry_t entry;
 	const char *demux_strategies[] = {"default", "reverse", "content",
@@ -646,13 +655,12 @@ setup_config (BaconVideoWidget *bvw)
 	bvw->priv->volume = -1;
 
 	/* Proxy configuration */
-	gc = gconf_client_get_default ();
-	if (gc == NULL)
+	if (bvw->priv->gc == NULL)
 		return;
 
 	memset (&entry, 0, sizeof (entry));
 
-	if (gconf_client_get_bool (gc, "/system/http_proxy/use_http_proxy", NULL) == FALSE)
+	if (gconf_client_get_bool (bvw->priv->gc, "/system/http_proxy/use_http_proxy", NULL) == FALSE)
 	{
 		if (xine_config_lookup_entry (bvw->priv->xine,
 				"input.http_proxy_host", &entry))
@@ -668,7 +676,7 @@ setup_config (BaconVideoWidget *bvw)
 	if (xine_config_lookup_entry (bvw->priv->xine,
 			"input.http_proxy_host", &entry))
 	{
-		entry.str_value = gconf_client_get_string (gc,
+		entry.str_value = gconf_client_get_string (bvw->priv->gc,
 				"/system/http_proxy/host", NULL);
 		xine_config_update_entry (bvw->priv->xine, &entry);
 	}
@@ -677,13 +685,13 @@ setup_config (BaconVideoWidget *bvw)
 	if (xine_config_lookup_entry (bvw->priv->xine,
 			"input.http_proxy_port", &entry))
 	{
-		entry.num_value = gconf_client_get_int (gc,
+		entry.num_value = gconf_client_get_int (bvw->priv->gc,
 				"/system/http_proxy/port", NULL);
 		xine_config_update_entry (bvw->priv->xine, &entry);
 	}
 
 	memset (&entry, 0, sizeof (entry));
-	if (gconf_client_get_bool (gc, "/system/http_proxy/use_authentication", NULL) == FALSE)
+	if (gconf_client_get_bool (bvw->priv->gc, "/system/http_proxy/use_authentication", NULL) == FALSE)
 	{
 		if (xine_config_lookup_entry (bvw->priv->xine,
 				"input.http_proxy_user", &entry))
@@ -703,9 +711,10 @@ setup_config (BaconVideoWidget *bvw)
 		if (xine_config_lookup_entry (bvw->priv->xine,
 				"input.http_proxy_user", &entry))
 		{
-			entry.str_value = gconf_client_get_string (gc,
-					"/system/http_proxy/authentication_user",
-					NULL);
+			entry.str_value = gconf_client_get_string
+				(bvw->priv->gc,
+				 "/system/http_proxy/authentication_user",
+				 NULL);
 			xine_config_update_entry (bvw->priv->xine, &entry);
 		}
 
@@ -713,9 +722,10 @@ setup_config (BaconVideoWidget *bvw)
 		if (xine_config_lookup_entry (bvw->priv->xine,
 				"input.http_proxy_password", &entry))
 		{
-			entry.str_value = gconf_client_get_string (gc,
-					"/system/http_proxy/authentication_password",
-					NULL);
+			entry.str_value = gconf_client_get_string
+				(bvw->priv->gc,
+				 "/system/http_proxy/authentication_password",
+				 NULL);
 			xine_config_update_entry (bvw->priv->xine, &entry);
 		}
 	}
@@ -735,6 +745,24 @@ setup_config_video (BaconVideoWidget *bvw)
 	}
 }
 
+static void
+setup_config_stream (BaconVideoWidget *bvw)
+{
+	int value, i;
+
+	if (bvw->priv->gc == NULL)
+		return;
+
+	for (i = 0; i < 2; i++)
+	{
+		value = gconf_client_get_int (bvw->priv->gc, video_props_str[i], NULL);
+		/* avoid black screens */
+		if (value == 0)
+			value = 65535 / 2;
+		xine_set_param (bvw->priv->stream, video_props[i], value);
+	}
+}
+
 static gboolean
 video_window_translate_point (BaconVideoWidget *bvw, int gui_x, int gui_y,
 		int *video_x, int *video_y)
@@ -958,9 +986,9 @@ bacon_video_widget_realize (GtkWidget *widget)
 	bvw->priv->screen = DefaultScreen (bvw->priv->display);
 
 	bvw->priv->vo_driver = load_video_out_driver (bvw, bvw->priv->null_out);
-	setup_config_video (bvw);
 	//FIXME
 	g_assert (bvw->priv->vo_driver != NULL);
+	setup_config_video (bvw);
 
 	if (bvw->priv->null_out == FALSE && bvw->priv->ao_driver != NULL)
 		bvw->priv->vis = xine_post_init (bvw->priv->xine,
@@ -969,6 +997,7 @@ bacon_video_widget_realize (GtkWidget *widget)
 
 	bvw->priv->stream = xine_stream_new (bvw->priv->xine,
 			bvw->priv->ao_driver, bvw->priv->vo_driver);
+	setup_config_stream (bvw);
 	bvw->priv->ev_queue = xine_event_new_queue (bvw->priv->stream);
 
 	/* Setup xine events */
@@ -1026,6 +1055,7 @@ bacon_video_widget_idle_signal (BaconVideoWidget *bvw)
 	return (queue_length > 0);
 }
 
+//FIXME this needs to be launched from the async handler
 static void
 xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 {
@@ -1036,7 +1066,7 @@ xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 	if (!(data->explanation))
 	{
 		D("xine_event_message: UI message without an explanation\n"
-				"type: %d", msg->type);
+				"type: %d", data->type);
 		return;
 	}
 
@@ -1065,7 +1095,7 @@ xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 		message = g_strdup_printf (_("The movie '%s' could not be read."), (char *) data + data->parameters);
 		break;
 	case XINE_MSG_LIBRARY_LOAD_ERROR:
-		message = g_strdup_printf (_("A problem occur while loading a library or a decoder (%s)."), (char *) data + data->parameters);
+		message = g_strdup_printf (_("A problem occured while loading a library or a decoder (%s)."), (char *) data + data->parameters);
 		break;
 	case XINE_MSG_ENCRYPTED_SOURCE:
 		message = g_strdup_printf (_("The source seems encrypted, and can't be read. Are you trying to play an encrypted DVD without libdvdcss?"));
@@ -1075,13 +1105,13 @@ xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 	if (message == NULL)
 	{
 		D("xine_event_message: unhandled error\n"
-				"type: %d", msg->type);
+				"type: %d", data->type);
 		return;
 	}
 
 	g_signal_emit (G_OBJECT (bvw),
 			bvw_table_signals[ERROR], 0,
-			message);
+			message, TRUE);
 
 	g_free (message);
 }
@@ -1481,7 +1511,7 @@ bacon_video_widget_get_backend_name (BaconVideoWidget *bvw)
 }
 
 gboolean
-bacon_video_widget_open (BaconVideoWidget *bvw, const gchar *mrl,
+bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 		GError **gerror)
 {
 	int error;
@@ -1493,6 +1523,8 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const gchar *mrl,
 	g_return_val_if_fail (bvw->priv->mrl == NULL, FALSE);
 
 	bvw->priv->mrl = g_strdup (mrl);
+	bvw->priv->video_fcc = 0;
+	bvw->priv->audio_fcc = 0;
 
 	error = xine_open (bvw->priv->stream, mrl);
 	if (error == 0)
@@ -1509,17 +1541,30 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const gchar *mrl,
 				&& xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_AUDIO_HANDLED) == FALSE))
 	{
-		uint32_t fourcc;
 		char *fourcc_str, *name;
 
 		g_signal_emit (G_OBJECT (bvw),
 				bvw_table_signals[GOT_METADATA], 0, NULL);
 
-		fourcc = xine_get_stream_info (bvw->priv->stream,
+		bvw->priv->video_fcc = xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_VIDEO_FOURCC);
-		fourcc_str = get_fourcc_string (fourcc);
+		fourcc_str = get_fourcc_string (bvw->priv->video_fcc);
 		name = g_strdup (xine_get_meta_info (bvw->priv->stream,
 				XINE_META_INFO_VIDEOCODEC));
+#ifdef HAVE_X86
+		/* Only change the audio_fcc if we're on x86 */
+		if (xine_get_stream_info
+				(bvw->priv->stream,
+				 XINE_STREAM_INFO_AUDIO_HANDLED) == FALSE)
+		{
+			bvw->priv->audio_fcc = xine_get_stream_info
+				(bvw->priv->stream,
+				 XINE_STREAM_INFO_AUDIO_FOURCC);
+		}
+#else
+		/* Reset video_fcc, and leave audio_fcc at 0 */
+		bvw->priv->video_fcc = 0;
+#endif
 
 		bacon_video_widget_close (bvw);
 
@@ -1708,7 +1753,7 @@ bacon_video_widget_set_logo_mode (BaconVideoWidget *bvw, gboolean logo_mode)
 }
 
 void
-bacon_video_widget_set_logo (BaconVideoWidget *bvw, gchar *filename)
+bacon_video_widget_set_logo (BaconVideoWidget *bvw, char *filename)
 {
 	g_return_if_fail (bvw != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET(bvw));
@@ -2121,12 +2166,12 @@ bacon_video_widget_set_visuals_quality (BaconVideoWidget *bvw,
 	switch (quality)
 	{
 	case VISUAL_SMALL:
-		fps = 25;
-		w = 160;
-		h = 120;
+		fps = 15;
+		w = 320;
+		h = 240;
 		break;
 	case VISUAL_NORMAL:
-		fps = 15;
+		fps = 25;
 		w = 320;
 		h = 240;
 		break;
@@ -2136,9 +2181,9 @@ bacon_video_widget_set_visuals_quality (BaconVideoWidget *bvw,
 		h = 480;
 		break;
 	case VISUAL_EXTRA_LARGE:
-		fps = 25;
-		w = 1024;
-		h = 768;
+		fps = 30;
+		w = 800;
+		h = 600;
 		break;
 	default:
 		/* shut up warnings */
@@ -2278,7 +2323,7 @@ bacon_video_widget_can_play (BaconVideoWidget *bvw, MediaType type)
 	}
 }
 
-G_CONST_RETURN gchar
+G_CONST_RETURN char
 **bacon_video_widget_get_mrls (BaconVideoWidget *bvw, MediaType type)
 {
 	char *plugin_id;
@@ -2297,7 +2342,7 @@ G_CONST_RETURN gchar
 	else
 		return NULL;
 
-	return (G_CONST_RETURN gchar **) xine_get_autoplay_mrls
+	return (G_CONST_RETURN char **) xine_get_autoplay_mrls
 		(bvw->priv->xine, plugin_id, &num_mrls);
 }
 
@@ -2406,6 +2451,30 @@ bacon_video_widget_set_scale_ratio (BaconVideoWidget *bvw, gfloat ratio)
 	gtk_window_resize (GTK_WINDOW (toplevel), new_w, new_h);
 }
 
+int
+bacon_video_widget_get_video_property (BaconVideoWidget *bvw,
+		BaconVideoWidgetVideoProperty type)
+{
+	g_return_val_if_fail (bvw != NULL, 65535 / 2);
+	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), 65535 / 2);
+	g_return_val_if_fail (bvw->priv->xine != NULL, 65535 / 2);
+
+	return xine_get_param (bvw->priv->stream, video_props[type]);
+}
+
+void
+bacon_video_widget_set_video_property (BaconVideoWidget *bvw,
+		BaconVideoWidgetVideoProperty type, int value)
+{
+	g_return_if_fail (bvw != NULL);
+	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+	g_return_if_fail (bvw->priv->xine != NULL);
+	g_return_if_fail ((value < 65535 && value > 0));
+
+	xine_set_param (bvw->priv->stream, video_props[type], value);
+	gconf_client_set_int (bvw->priv->gc, video_props_str[type], value, NULL);
+}
+
 static void
 bacon_video_widget_get_metadata_string (BaconVideoWidget *bvw, BaconVideoWidgetMetadataType type,
 		GValue *value)
@@ -2452,8 +2521,8 @@ bacon_video_widget_get_metadata_string (BaconVideoWidget *bvw, BaconVideoWidgetM
 }
 
 static void
-bacon_video_widget_get_metadata_int (BaconVideoWidget *bvw, BaconVideoWidgetMetadataType type,
-		GValue *value)
+bacon_video_widget_get_metadata_int (BaconVideoWidget *bvw,
+		BaconVideoWidgetMetadataType type, GValue *value)
 {
 	int integer = 0;
 
@@ -2478,6 +2547,9 @@ bacon_video_widget_get_metadata_int (BaconVideoWidget *bvw, BaconVideoWidgetMeta
 		integer = xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_VIDEO_HEIGHT);
 		break;
+	case BVW_INFO_VIDEO_FOURCC:
+		integer = (guint32) bvw->priv->video_fcc;
+		break;
 	case BVW_INFO_FPS:
 		if (xine_get_stream_info (bvw->priv->stream,
 					XINE_STREAM_INFO_FRAME_DURATION) != 0)
@@ -2493,6 +2565,9 @@ bacon_video_widget_get_metadata_int (BaconVideoWidget *bvw, BaconVideoWidgetMeta
 		integer = xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_AUDIO_BITRATE) / 1000;
 		break;
+	 case BVW_INFO_AUDIO_FOURCC:
+		integer = (guint32) bvw->priv->audio_fcc;
+		break;
 	 default:
 		g_assert_not_reached ();
 	 }
@@ -2536,11 +2611,13 @@ bacon_video_widget_get_metadata_bool (BaconVideoWidget *bvw,
 }
 
 void
-bacon_video_widget_get_metadata (BaconVideoWidget *bvw, BaconVideoWidgetMetadataType type, GValue *value)
+bacon_video_widget_get_metadata (BaconVideoWidget *bvw,
+		BaconVideoWidgetMetadataType type, GValue *value)
 {
 	g_return_if_fail (bvw != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 	g_return_if_fail (bvw->priv->xine != NULL);
+	g_return_if_fail (value != NULL);
 
 	switch (type)
 	{
@@ -2554,8 +2631,10 @@ bacon_video_widget_get_metadata (BaconVideoWidget *bvw, BaconVideoWidgetMetadata
 	case BVW_INFO_DURATION:
 	case BVW_INFO_DIMENSION_X:
 	case BVW_INFO_DIMENSION_Y:
+	case BVW_INFO_VIDEO_FOURCC:
 	case BVW_INFO_FPS:
 	case BVW_INFO_BITRATE:
+	case BVW_INFO_AUDIO_FOURCC:
 		bacon_video_widget_get_metadata_int (bvw, type, value);
 		break;
 	case BVW_INFO_HAS_VIDEO:
@@ -2682,18 +2761,6 @@ bacon_video_widget_can_get_frames (BaconVideoWidget *bvw, GError **error)
 		return FALSE;
 	}
 
-	if (bvw->priv->using_vfx == FALSE
-			&& (xine_get_stream_info (bvw->priv->stream,
-				XINE_STREAM_INFO_VIDEO_WIDTH) == 0
-			|| xine_get_stream_info (bvw->priv->stream,
-				XINE_STREAM_INFO_VIDEO_HEIGHT) == 0))
-	{
-		g_set_error (error, 0, 0,
-				_("Height or width of the video is 0. "
-					"Please file a bug."));
-		return FALSE;
-	}
-
 	return TRUE;
 }
 
@@ -2712,6 +2779,9 @@ bacon_video_widget_get_current_frame (BaconVideoWidget *bvw)
 			&ratio, &format, NULL) == 0)
 		return NULL;
 
+	if (width == 0 || height == 0)
+		return NULL;
+
 	yuv = malloc ((width + 8) * (height + 1) * 2);
 	if (yuv == NULL)
 		return NULL;
