@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget.h,v 1.22 2003/05/11 13:30:29 hadess Exp $
+ * $Id: bacon-video-widget.h,v 1.27 2003/06/21 01:49:31 hadess Exp $
  *
  * the xine engine in a widget - header
  */
@@ -55,13 +55,20 @@ typedef enum {
 	BVW_INFO_DIMENSION_X,
 	BVW_INFO_DIMENSION_Y,
 	BVW_INFO_VIDEO_CODEC,
+	BVW_INFO_VIDEO_FOURCC,
 	BVW_INFO_FPS,
 	/* Audio */
 	BVW_INFO_HAS_AUDIO,
 	BVW_INFO_BITRATE,
-	BVW_INFO_AUDIO_CODEC
+	BVW_INFO_AUDIO_CODEC,
+	BVW_INFO_AUDIO_FOURCC,
 } BaconVideoWidgetMetadataType;
 
+typedef enum {
+	BVW_VIDEO_BRIGHTNESS,
+	BVW_VIDEO_CONTRAST,
+} BaconVideoWidgetVideoProperty;
+
 #define BACON_VIDEO_WIDGET(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), bacon_video_widget_get_type (), BaconVideoWidget))
 #define BACON_VIDEO_WIDGET_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), bacon_video_widget_get_type (), BaconVideoWidgetClass))
 #define BACON_IS_VIDEO_WIDGET(obj)           (G_TYPE_CHECK_INSTANCE_TYPE (obj, bacon_video_widget_get_type ()))
@@ -77,7 +84,8 @@ typedef struct {
 typedef struct {
 	GtkVBoxClass parent_class;
 
-	void (*error) (GtkWidget *bvw, const char *message);
+	void (*error) (GtkWidget *bvw, const char *message,
+                       gboolean playback_stopped);
 	void (*eos) (GtkWidget *bvw);
 	void (*got_metadata) (GtkWidget *bvw);
 	void (*title_change) (GtkWidget *bvw, const char *title);
@@ -124,21 +132,21 @@ void bacon_video_widget_set_proprietary_plugins_path
 						 (BaconVideoWidget *bvw,
 				                  const char *path);
 
-void bacon_video_widget_set_fullscreen           (BaconVideoWidget *bvw,
-						  gboolean fullscreen);
-gboolean bacon_video_widget_is_fullscreen        (BaconVideoWidget *bvw);
-
 gboolean bacon_video_widget_can_set_volume       (BaconVideoWidget *bvw);
 void bacon_video_widget_set_volume               (BaconVideoWidget *bvw,
 						  int volume);
 int bacon_video_widget_get_volume                (BaconVideoWidget *bvw);
 
+void bacon_video_widget_set_fullscreen		 (BaconVideoWidget *bvw,
+						  gboolean fullscreen);
+
 void bacon_video_widget_set_show_cursor          (BaconVideoWidget *bvw,
 						  gboolean use_cursor);
 gboolean bacon_video_widget_get_show_cursor      (BaconVideoWidget *bvw);
 
 void bacon_video_widget_set_media_device	 (BaconVideoWidget *bvw,
 						  const char *path);
+gboolean bacon_video_widget_get_auto_resize (BaconVideoWidget *bvw);
 void bacon_video_widget_set_auto_resize		 (BaconVideoWidget *bvw,
 						  gboolean auto_resize);
 
@@ -158,6 +166,14 @@ void bacon_video_widget_toggle_aspect_ratio      (BaconVideoWidget *bvw);
 void bacon_video_widget_set_scale_ratio          (BaconVideoWidget *bvw,
 						  gfloat ratio);
 
+int bacon_video_widget_get_video_property	 (BaconVideoWidget *bvw,
+						  BaconVideoWidgetVideoProperty
+						  type);
+void bacon_video_widget_set_video_property	 (BaconVideoWidget *bvw,
+						  BaconVideoWidgetVideoProperty
+						  type,
+						  int value);
+
 int bacon_video_widget_get_position              (BaconVideoWidget *bvw);
 int bacon_video_widget_get_current_time          (BaconVideoWidget *bvw);
 int bacon_video_widget_get_stream_length         (BaconVideoWidget *bvw);
