@@ -7,6 +7,8 @@
 
 /* #define THUMB_DEBUG */
 
+#define MIN_LEN_FOR_SEEK 25000
+
 static void
 print_usage (void)
 {
@@ -164,7 +166,7 @@ int main (int argc, char *argv[])
 	GError *err = NULL;
 	BaconVideoWidget *bvw;
 	GdkPixbuf *pixbuf;
-	int i;
+	int i, length;
 
 	g_thread_init (NULL);
 
@@ -182,6 +184,9 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
+	//FIXME create a new thread that will watch over these 2 next
+	//calls so that we know they won't take longer than 30 secs
+
 	if (bacon_video_widget_open (bvw, argv[1], &err) == FALSE)
 	{
 		g_print ("totem-video-thumbnailer couln't open file '%s'\n"
@@ -191,8 +196,7 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	/* A 3rd into the file */
-	bacon_video_widget_play (bvw, (int) (65535 / 3), 0, &err);
+	bacon_video_widget_play (bvw, 0, 0, &err);
 	if (err != NULL)
 	{
 		g_print ("totem-video-thumbnailer couln't play file: '%s'\n"
@@ -202,6 +206,17 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
+	/* A 3rd into the file */
+	length = bacon_video_widget_get_stream_length (bvw);
+	if (length > MIN_LEN_FOR_SEEK)
+	{
+		if (bacon_video_widget_play
+				(bvw, 0, (int) (65535 / 3), NULL) == FALSE)
+		{
+			bacon_video_widget_play (bvw, 0, 0, NULL);
+		}
+	}
+
 	if (bacon_video_widget_can_get_frames (bvw, &err) == FALSE)
 	{
 		g_print ("totem-video-thumbnailer: '%s' isn't thumbnailable\n"
