@@ -103,6 +103,7 @@ static void playlist_changed_cb (GtkWidget *playlist, Totem *totem);
 static gboolean totem_is_media (const char *mrl); 
 static void show_controls (Totem *totem, gboolean visible, gboolean fullscreen_behaviour);
 static gboolean totem_is_fullscreen (Totem *totem);
+static void play_pause_set_label (Totem *totem, TotemStates state);
 
 static void
 long_action (void)
@@ -182,7 +183,7 @@ totem_action_error (char *msg, Totem *totem)
 		parent = totem->win;
 
 	error_dialog =
-		gtk_message_dialog_new (GTK_WINDOW (totem->win),
+		gtk_message_dialog_new (GTK_WINDOW (parent),
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_OK,
@@ -201,10 +202,61 @@ totem_action_error (char *msg, Totem *totem)
 		totem->buffer_dialog = NULL;
 		totem->buffer_label = NULL;
 	}
-
+        
 	gtk_widget_show (error_dialog);
 }
 
+#ifdef HAVE_X86
+static gboolean
+totem_action_error_try_download (char *msg, Totem *totem)
+{
+	GtkWidget *error_dialog;
+	GValue value = { 0, };
+	guint32 audio_fcc, video_fcc;
+	int res;
+
+	bacon_video_widget_get_metadata (totem->bvw,
+			BVW_INFO_VIDEO_FOURCC, &value);
+	video_fcc = (guint32) g_value_get_int (&value);
+	g_value_unset (&value);
+
+	bacon_video_widget_get_metadata (totem->bvw,
+			BVW_INFO_AUDIO_FOURCC, &value);
+	audio_fcc = (guint32) g_value_get_int (&value);
+
+	if (audio_fcc == 0 && video_fcc == 0)
+	{
+		totem_action_error (msg, totem);
+		return;
+	}
+
+	error_dialog =
+		gtk_message_dialog_new (GTK_WINDOW (totem->win),
+				GTK_DIALOG_MODAL,
+				GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_NONE,
+				"%s", msg);
+	gtk_dialog_add_buttons (GTK_DIALOG (error_dialog),
+			GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+			_("Download"), GTK_RESPONSE_ACCEPT,
+			NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (error_dialog),
+			GTK_RESPONSE_OK);
+	gtk_window_set_modal (GTK_WINDOW (error_dialog), TRUE);
+	res = gtk_dialog_run (GTK_DIALOG (error_dialog));
+	gtk_widget_destroy (error_dialog);
+
+	if (res != GTK_RESPONSE_ACCEPT)
+		return FALSE;
+
+	if (totem_download_from_fourcc (GTK_WINDOW (totem->win),
+				video_fcc, audio_fcc) < 0)
+		return FALSE;
+
+	return TRUE;
+}
+#endif
+
 void
 totem_action_error_and_exit (char *msg, Totem *totem)
 {
@@ -369,7 +421,7 @@ totem_action_set_mrl_and_play (Totem *totem, char *mrl)
 }
 
 void
-totem_action_play_media (Totem *totem, MediaType type)
+totem_action_load_media (Totem *totem, MediaType type)
 {
 	const char **mrls;
 	char *mrl;
@@ -391,6 +443,14 @@ totem_action_play_media (Totem *totem, MediaType type)
 	}
 
 	totem_action_open_files (totem, (char **)mrls, FALSE);
+}
+
+void
+totem_action_play_media (Totem *totem, MediaType type)
+{
+	char *mrl;
+
+	totem_action_load_media (totem, type);
 	mrl = gtk_playlist_get_current_mrl (totem->playlist);
 	totem_action_set_mrl_and_play (totem, mrl);
 	g_free (mrl);
@@ -466,10 +526,11 @@ totem_action_fullscreen_toggle (Totem *totem)
 			{
 				totem->controls_visibility =
 					TOTEM_CONTROLS_VISIBLE;
-				show_controls (totem, TRUE, FALSE);
+				show_controls (totem, TRUE, TRUE);
 			} else {
 				totem->controls_visibility =
 					TOTEM_CONTROLS_HIDDEN;
+				show_controls (totem, FALSE, TRUE);
 			}
 		}
 	} else {
@@ -480,12 +541,8 @@ totem_action_fullscreen_toggle (Totem *totem)
 		gtk_window_fullscreen (GTK_WINDOW(totem->win));
 		scrsaver_disable (totem->scr);
 
-		if (totem->controls_visibility == TOTEM_CONTROLS_VISIBLE) {
-			totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
-			show_controls (totem, FALSE, FALSE);
-		} else {
-			totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
-		}
+		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
+		show_controls (totem, FALSE, TRUE);
 	}
 }
 
@@ -715,22 +772,15 @@ totem_action_set_mrl (Totem *totem, const char *mrl)
 
 		update_mrl_label (totem, NULL);
 	} else {
-		char *name;
-		gboolean caps, custom;
+		gboolean caps, first_try;
 		GError *err = NULL;
 
+		first_try = TRUE;
 		bacon_video_widget_set_logo_mode (totem->bvw, FALSE);
 
+try_open_again:
 		retval = bacon_video_widget_open (totem->bvw, mrl, &err);
 		totem->mrl = g_strdup (mrl);
-		custom = FALSE;
-		name = totem_get_nice_name_for_stream (totem);
-		if (name == NULL)
-		{
-			name = gtk_playlist_get_current_title
-				(totem->playlist,
-				 &custom);
-		}
 
 		/* Play/Pause */
 		gtk_widget_set_sensitive (totem->pp_button, TRUE);
@@ -740,13 +790,6 @@ totem_action_set_mrl (Totem *totem, const char *mrl)
 		gtk_widget_set_sensitive (widget, TRUE);
 		gtk_widget_set_sensitive (totem->fs_pp_button, TRUE);
 
-		update_mrl_label (totem, name);
-		if (custom == FALSE)
-		{
-			gtk_playlist_set_title
-				(GTK_PLAYLIST (totem->playlist), name);
-		}
-
 		/* Seek bar */
 		update_seekable (totem, FALSE);
 
@@ -764,18 +807,33 @@ totem_action_set_mrl (Totem *totem, const char *mrl)
 		/* Set the playlist */
 		gtk_playlist_set_playing (totem->playlist, retval);
 
-		g_free (name);
-
-		if (retval == FALSE)
+		if (retval == FALSE && first_try == TRUE)
 		{
 			char *msg;
+			gboolean try_again;
 
 			msg = g_strdup_printf(_("Totem could not play '%s'.\n"
 						"Reason: %s."),
 					mrl,
 					err->message);
-			totem_action_error (msg, totem);
+#ifdef HAVE_X86
+			try_again = totem_action_error_try_download
+				(msg, totem);
 			g_free (msg);
+			first_try = FALSE;
+			if (try_again != FALSE)
+			{
+				g_error_free (err);
+				err = NULL;
+
+				goto try_open_again;
+			}
+#else
+			totem_action_error (msg, totem);
+#endif
+			g_free (msg);
+		} else if (first_try == FALSE) {
+			retval = FALSE;
 		}
 	}
 	update_buttons (totem);
@@ -1119,9 +1177,25 @@ on_channels_change_event (BaconVideoWidget *bvw, Totem *totem)
 static void
 on_got_metadata_event (BaconVideoWidget *bvw, Totem *totem)
 {
-	bacon_video_widget_properties_update
+        char *name = NULL;
+	gboolean custom;
+
+        bacon_video_widget_properties_update
 		(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
 		 totem->bvw, FALSE);
+        name = totem_get_nice_name_for_stream (totem);
+	if (name == NULL)
+	{
+		name = gtk_playlist_get_current_title
+			(totem->playlist,
+			 &custom);
+	}
+
+	update_mrl_label (totem, name);
+
+	if (custom == FALSE)
+		gtk_playlist_set_title (GTK_PLAYLIST (totem->playlist), name);
+	g_free (name);
 }
 
 static int
@@ -1141,8 +1215,11 @@ on_buffering_cancel_event (GtkWidget *dialog, int response, Totem *totem)
 }
 
 static void
-on_error_event (BaconVideoWidget *bvw, const char *message, Totem *totem)
+on_error_event (BaconVideoWidget *bvw, char *message,
+                gboolean playback_stopped, Totem *totem)
 {
+        if (playback_stopped)
+            play_pause_set_label (totem, STATE_STOPPED);
 	totem_action_error (message, totem);
 }
 
@@ -1192,8 +1269,6 @@ on_buffering_event (BaconVideoWidget *bvw, int percentage, Totem *totem)
 
 		gtk_widget_show_all (totem->buffer_dialog);
 		long_action ();
-
-		//FIXME connect cancel
 	} else if (percentage == 100) {
 		if (totem->buffer_dialog != NULL)
 			gtk_widget_destroy (totem->buffer_dialog);
@@ -1406,13 +1481,13 @@ totem_action_open_files (Totem *totem, char **list, gboolean ignore_first)
 			}
 			if (strcmp (list[i], "dvd:") == 0)
 			{
-				totem_action_play_media (totem, MEDIA_DVD);
+				totem_action_load_media (totem, MEDIA_DVD);
 				continue;
 			} else if (strcmp (list[i], "vcd:") == 0) {
-				totem_action_play_media (totem, MEDIA_VCD);
+				totem_action_load_media (totem, MEDIA_VCD);
 				continue;
 			} else if (strcmp (list[i], "cd:") == 0) {
-				totem_action_play_media (totem, MEDIA_CDDA);
+				totem_action_load_media (totem, MEDIA_CDDA);
 				continue;
 			} else if (gtk_playlist_add_mrl (totem->playlist,
 						filename, NULL) == TRUE)
@@ -1671,19 +1746,23 @@ show_controls (Totem *totem, gboolean visible, gboolean fullscreen_behaviour)
 		gtk_widget_show (menubar);
 		gtk_widget_show (controlbar);
 		gtk_widget_show (statusbar);
-		gtk_widget_show (item);
+		gtk_widget_hide (item);
 		gtk_container_set_border_width (GTK_CONTAINER (bvw_vbox), 1);
 	} else {
 		gtk_widget_hide (menubar);
 		gtk_widget_hide (controlbar);
 		gtk_widget_hide (statusbar);
-		gtk_widget_hide (item);
+		 /* We won't show controls in fullscreen */
+		if (totem->controls_visibility == TOTEM_CONTROLS_FULLSCREEN)
+			gtk_widget_hide (item);
+		else
+			gtk_widget_show (item);
 		gtk_container_set_border_width (GTK_CONTAINER (bvw_vbox), 0);
 	}
 	
 	/* If we are called from fullscreen handlers
 	we do not handle the window's size */
-	if (!fullscreen_behaviour)
+	if (fullscreen_behaviour)
 		return;
 	
 	if (totem->controls_visibility == TOTEM_CONTROLS_HIDDEN) {
@@ -1719,7 +1798,7 @@ on_show_controls1_activate (GtkCheckMenuItem *checkmenuitem, Totem *totem)
 	} else {
 		totem->controls_visibility = TOTEM_CONTROLS_HIDDEN;
 	}
-	show_controls (totem, show, TRUE);
+	show_controls (totem, show, FALSE);
 }
 
 static void
@@ -1825,9 +1904,16 @@ screenshot_make_filename (Totem *totem)
 
 	/* Test if we have a desktop directory */
 	fullpath = g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (),
-			".gnome-desktop", NULL);
+			"Desktop", NULL);
 	desktop_exists = g_file_test (fullpath, G_FILE_TEST_EXISTS);
 	g_free (fullpath);
+	if (desktop_exists == FALSE)
+	{
+		fullpath = g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (),
+				".gnome-desktop", NULL);
+		desktop_exists = g_file_test (fullpath, G_FILE_TEST_EXISTS);
+		g_free (fullpath);
+	}
 
 	if (on_desktop == TRUE)
 	{
@@ -2290,8 +2376,7 @@ on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event,
 		Totem *totem)
 {
 	int width;
-	gint pointer_x, pointer_y;
-	GdkModifierType state;
+
 	if (totem_is_fullscreen (totem) == FALSE) 
 		return FALSE;
 
@@ -2299,6 +2384,7 @@ on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event,
 		return FALSE;
 
 	totem->popup_in_progress = TRUE;
+
 	if (totem->popup_timeout != 0)
 	{
 		gtk_timeout_remove (totem->popup_timeout);
@@ -2322,7 +2408,6 @@ on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event,
 	totem->popup_timeout = gtk_timeout_add (5000,
 			(GtkFunction) popup_hide, totem);
 	totem->popup_in_progress = FALSE;
-	gdk_window_get_pointer (widget->window, &pointer_x, &pointer_y, &state);
 
 	return FALSE;
 }
@@ -2713,7 +2798,21 @@ create_submenu (Totem *totem, GList *list, int current, gboolean is_lang)
 	GSList *group = NULL;
 
 	if (list == NULL)
-		return NULL;
+	{
+		menu = gtk_menu_new ();
+		if (is_lang == FALSE)
+		{
+			group = add_item_to_menu (totem, menu, _("None"),
+					current, -2, is_lang, group);
+		}
+
+		group = add_item_to_menu (totem, menu, _("Auto"),
+				current, -1, is_lang, group);
+
+		gtk_widget_show (menu);
+
+		return menu;
+	}
 
 	i = 0;
 
@@ -2724,6 +2823,12 @@ create_submenu (Totem *totem, GList *list, int current, gboolean is_lang)
 		if (menu == NULL)
 		{
 			menu = gtk_menu_new ();
+			if (is_lang == FALSE)
+			{
+				group = add_item_to_menu (totem, menu,
+						_("None"), current, -2,
+						is_lang, group);
+			}
 			group = add_item_to_menu (totem, menu, _("Auto"),
 					current, -1, is_lang, group);
 		}
@@ -2746,23 +2851,23 @@ update_dvd_menu_sub_lang (Totem *totem)
 	GtkWidget *item, *submenu;
 	gboolean playing_dvd;
 	GtkWidget *lang_menu, *sub_menu;
+	GList *list;
+	int current;
 
 	lang_menu = NULL;
 	sub_menu = NULL;
 
-	playing_dvd = totem_playing_dvd (totem);
-
-	if (playing_dvd != FALSE)
+	list = bacon_video_widget_get_languages (totem->bvw);
+	if (list != NULL)
 	{
-		GList *list;
-		int current;
-
-		list = bacon_video_widget_get_languages (totem->bvw);
 		current = bacon_video_widget_get_language (totem->bvw);
 		lang_menu = create_submenu (totem, list, current, TRUE);
 		totem_g_list_deep_free (list);
+	}
 
-		list = bacon_video_widget_get_subtitles (totem->bvw);
+	list = bacon_video_widget_get_subtitles (totem->bvw);
+	if (list != NULL)
+	{
 		current = bacon_video_widget_get_subtitle (totem->bvw);
 		sub_menu = create_submenu (totem, list, current, FALSE);
 		totem_g_list_deep_free (list);
@@ -3240,7 +3345,8 @@ totem_setup_recent (Totem *totem)
 
 	totem->recent_view = egg_recent_view_gtk_new (menu, menu_item);
 	egg_recent_view_gtk_show_icons (EGG_RECENT_VIEW_GTK
-						(totem->recent_view), FALSE);
+			(totem->recent_view), FALSE);
+	egg_recent_model_set_limit (totem->recent_model, 5);
 	egg_recent_view_set_model (EGG_RECENT_VIEW (totem->recent_view),
 			totem->recent_model);
 	egg_recent_view_gtk_set_trailing_sep (totem->recent_view, TRUE);
@@ -3409,12 +3515,8 @@ main (int argc, char **argv)
 
 	/* IPC stuff */
 	totem->conn = bacon_message_connection_new (GETTEXT_PACKAGE);
-	if (bacon_message_connection_get_is_server (totem->conn) == TRUE)
+	if (bacon_message_connection_get_is_server (totem->conn) == FALSE)
 	{
-		bacon_message_connection_set_callback (totem->conn,
-				(BaconMessageReceivedFunc)
-				totem_message_connection_receive_cb, totem);
-	} else {
 		process_command_line (totem->conn, argc, argv);
 		bacon_message_connection_free (totem->conn);
 		g_free (totem);
@@ -3519,6 +3621,13 @@ main (int argc, char **argv)
 		totem_action_restore_pl (totem);
 	}
 
+	if (bacon_message_connection_get_is_server (totem->conn) == TRUE)
+	{
+		bacon_message_connection_set_callback (totem->conn,
+				(BaconMessageReceivedFunc)
+				totem_message_connection_receive_cb, totem);
+	}
+
 #ifdef HAVE_REMOTE
 	totem->remote = totem_remote_new ();
 	g_signal_connect (totem->remote, "button_pressed",
